/*
 * File:       <compressDEM.C>
 * Author:     <Remigiusz Trzaska>
 * e-mail:     <remek@npac.syr.edu>
 * Created:    <Mon Apr  7 1997>
 * Time-stamp: <97/04/09 15:09:44 remek>
 * Copyright Remigiusz Trzaska 1997 NPAC, Syracuse University
 *
 */

#include "Memory.h"
#include <stdio.h>
#include "wv.h"

int
main(int argc, char* argv[])
{
   //check the args
   if(argc!=4)
   {
      fprintf(stderr, "Usage: %s infile outfile quality\n"
	      "\twhere\tinfile is a pgm image file\n"
	      "\t\toutfile is a name for the output file\n"
	      "\t\tquality is a number 1..11\n\n", argv[0]);
      return(-1);
   }
   FILE *inFile = fopen(argv[1], "rb");
   if(!inFile)
   {
      fprintf(stderr, "Cannot read the file %s\nExiting...\n\n", argv[1]);
      return(-1);
   }
   FILE *outFile = fopen(argv[2], "wb");
   if(!outFile)
   {
      fprintf(stderr, "Cannot open the file %s for writing\nExiting...\n\n",
	      argv[2]);
      return(-1);
   }
   //read the image
   char junk;
   fgetc(inFile);
   fgetc(inFile);
   junk = fgetc(inFile);
   int dimx = junk * 256;
   junk = fgetc(inFile);
   dimx += junk;
   int dimy = dimx;

   for(int i=0; i<4; fgetc(inFile), i++);

   float* demData;
   NewVector(demData, dimx*dimy, float);

   for(i = 0; i<dimy; i++)
      for(int j = 0; j<dimx; j++)
      {
	 junk = fgetc(inFile);
	 demData[i*dimx + dimx - j - 1] = junk*256.0;
	 junk = fgetc(inFile);
	 demData[i*dimx + dimx - j - 1] += junk;
      }

   int aMin = 10000;
   int aMax = -10000;
   for(i =0; i<dimx*dimy; i++)
   {
      if(demData[i]>aMax)
	 aMax = demData[i];
      if(demData[i]<aMin)
	 aMin = demData[i];
   }

   FILE *debugFile = fopen("original.pgm", "wb");
   fprintf(debugFile, "P5\n%d %d\n255\n", dimx, dimy);
   for(i=0; i<dimy; i++)
      for(int j=0; j<dimx; j++)
      {
	 float scaled = (demData[i*dimx+j]-aMin)/((float)(aMax-aMin))*255.0;
	 fprintf(debugFile, "%c", (char)scaled);
      }
   
   fclose(debugFile);

   //compress the image

   int quality = atoi(argv[3]);
   int usedLength;
   unsigned char* stream;

   NewVector(stream, dimx*dimy*sizeof(float), unsigned char);
   usedLength = dimx*dimy*sizeof(float);

   fprintf(stderr, "Diagnostics: aMin=%d, aMax=%d\n", aMin, aMax);

   ewv_t* compressor = new_ewv(dimy, dimx, aMin, aMax, aMax - aMin + 1);
   
   int returnedQuality = enc_ewv(compressor,
				 demData,
				 stream,
				 &usedLength,
				 quality);

   if(returnedQuality<0)
   {
      fprintf(stderr, "Compression failed! Returned stream length is %d\n",
	      usedLength);
      return(1);
   }

   delete_ewv(&compressor);

   //save the compressed stream
   fputc(dimy/256, outFile);
   fputc(dimy%256, outFile);
   fputc(dimx/256, outFile);
   fputc(dimx%256, outFile);
   fputc(aMin/256, outFile);
   fputc(aMin%256, outFile);
   fputc(aMax/256, outFile);
   fputc(aMax%256, outFile);
   fwrite(stream, sizeof(unsigned char), usedLength, outFile); 
   
   fclose(outFile);
   fclose(inFile);


   //uncompress and save in order to test
   delete demData;
   NewVector(demData, dimx*dimy, float);
   compressor = new_ewv(dimy, dimx, aMin, aMax, aMax - aMin + 1);
   returnedQuality = dec_ewv(compressor,
				 stream, //for header
				 &usedLength,
				 demData,
				 quality,
				 /*scale*/1);
   
   debugFile = fopen("compressed.pgm", "wb");
   fprintf(debugFile, "P5\n%d %d\n255\n", dimx, dimy);
   for(i=0; i<dimy; i++)
      for(int j=0; j<dimx; j++)
      {
	 float scaled = (demData[i*dimx+j]-aMin)/((float)(aMax-aMin))*255.0;
	 fprintf(debugFile, "%c", (char)scaled);
      }
   
   fclose(debugFile);

}

